# Generated by Django 2.2.24 on 2021-08-13 12:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('edx_name_affirmation', '0002_verifiednameconfig'),
    ]

    operations = [
        migrations.AddField(
            model_name='verifiedname',
            name='status',
            field=models.CharField(choices=[('pending', 'pending'), ('submitted', 'submitted'), ('approved', 'approved'), ('denied', 'denied')], default='pending', max_length=32),
        ),
    ]
