# Generated by Django 2.2.24 on 2021-08-30 20:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('edx_name_affirmation', '0005_remove_verifiedname_is_verified'),
    ]

    operations = [
        migrations.AlterField(
            model_name='verifiedname',
            name='proctored_exam_attempt_id',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='verifiedname',
            name='verification_attempt_id',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
    ]
