# Generated by Django 2.2.24 on 2021-08-18 10:27

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('edx_name_affirmation', '0004_auto_20210816_0958'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='verifiedname',
            name='is_verified',
        ),
    ]
