# Generated by Django 2.2.24 on 2021-06-15 09:05

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='VerifiedName',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('verified_name', models.CharField(db_index=True, max_length=255)),
                ('profile_name', models.CharField(max_length=255, null=True)),
                ('verification_attempt_id', models.PositiveIntegerField(null=True)),
                ('proctored_exam_attempt_id', models.PositiveIntegerField(null=True)),
                ('is_verified', models.BooleanField(default=False)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'verified name',
                'db_table': 'nameaffirmation_verifiedname',
            },
        ),
    ]
