# Generated by Django 3.2.5 on 2021-07-19 09:24

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('edx_name_affirmation', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='VerifiedNameConfig',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('change_date', models.DateTimeField(auto_now_add=True, verbose_name='Change date')),
                ('enabled', models.BooleanField(default=False, verbose_name='Enabled')),
                ('use_verified_name_for_certs', models.BooleanField(default=False)),
                ('changed_by', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL, verbose_name='Changed by')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='verified_name_config', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'verified name config',
                'db_table': 'nameaffirmation_verifiednameconfig',
            },
        ),
    ]
